@@ -1,6 +1,6 @@
 package DBIx::Custom;
 
-our $VERSION = '0.1642';
+our $VERSION = '0.1643';
 
 use 5.008001;
 use strict;
@@ -25,7 +25,8 @@ __PACKAGE__->attr(
     query_builder => sub { DBIx::Custom::QueryBuilder->new },
     result_class  => 'DBIx::Custom::Result',
     base_table    => sub { DBIx::Custom::Table->new(dbi => shift) },
-    safety_column_name => sub { qr/^[\w\.]*$/ }
+    safety_column_name => sub { qr/^[\w\.]*$/ },
+    stash => sub { {} }
 );
 
 __PACKAGE__->attr(
@@ -80,54 +81,45 @@ sub apply_filter {
     
     # Create filters
     my $usage = "Usage: \$dbi->apply_filter(" .
-                "TABLE, COLUMN1, {in => INFILTER1, out => OUTFILTER1}, " .
-                "COLUMN2, {in => INFILTER2, out => OUTFILTER2}, ...)";
+                "TABLE, COLUMN1, {in => INFILTER1, out => OUTFILTER1, end => ENDFILTER1}, " .
+                "COLUMN2, {in => INFILTER2, out => OUTFILTER2, end => ENDFILTER2}, ...)";
 
     for (my $i = 0; $i < @cinfos; $i += 2) {
         
         # Column
         my $column = $cinfos[$i];
         
-        # Filter
-        my $filter = $cinfos[$i + 1] || {};
-        croak $usage unless  ref $filter eq 'HASH';
-        foreach my $ftype (keys %$filter) {
-            croak $usage unless $ftype eq 'in' || $ftype eq 'out'; 
+        # Filter info
+        my $finfo = $cinfos[$i + 1] || {};
+        croak $usage unless  ref $finfo eq 'HASH';
+        foreach my $ftype (keys %$finfo) {
+            croak $usage unless $ftype eq 'in' || $ftype eq 'out'
+                             || $ftype eq 'end'; 
         }
-        my $in_filter = $filter->{in};
-        my $out_filter = $filter->{out};
         
-        # Out filter
-        if (ref $out_filter eq 'CODE') {
-            $self->{filter}{out}{$table}{$column}
-              = $out_filter;
-            $self->{filter}{out}{$table}{"$table.$column"}
-              = $out_filter;
-        }
-        elsif (defined $out_filter) {
-            croak qq{Filter "$out_filter" is not registered}
-              unless exists $self->filters->{$out_filter};
+        foreach my $way (qw/in out end/) {
+            my $filter = $finfo->{$way};
             
-            $self->{filter}{out}{$table}{$column}
-              = $self->filters->{$out_filter};
-            $self->{filter}{out}{$table}{"$table.$column"}
-              = $self->filters->{$out_filter};
-        }
-        
-        # In filter
-        if (ref $in_filter eq 'CODE') {
-            $self->{filter}{in}{$table}{$column}
-              = $in_filter;
-            $self->{filter}{in}{$table}{"$table.$column"}
-              = $in_filter;
-        }
-        elsif (defined $in_filter) {
-            croak qq{Filter "$in_filter" is not registered}
-              unless exists $self->filters->{$in_filter};
-            $self->{filter}{in}{$table}{$column}
-              = $self->filters->{$in_filter};
-            $self->{filter}{in}{$table}{"$table.$column"}
-              = $self->filters->{$in_filter};
+            # State
+            my $state = !exists $finfo->{$way} ? 'not_exists'
+                      : !defined $filter        ? 'not_defined'
+                      : ref $filter eq 'CODE'   ? 'code'
+                      : 'name';
+            
+            next if $state eq 'not_exists';
+            
+            # Check filter
+            croak qq{Filter "$filter" is not registered}
+              if  $state eq 'name'
+               && ! exists $self->filters->{$filter};
+            
+            # Filter
+            my $f = $state eq 'not_defined' ? undef
+                  : $state eq 'code'        ? $filter
+                  : $self->filters->{$filter};
+            $self->{filter}{$way}{$table}{$column} = $f;
+            $self->{filter}{$way}{$table}{"$table.$column"} = $f;
+            $self->{filter}{$way}{$table}{"${table}__$column"} = $f;
         }
     }
     
@@ -191,7 +183,10 @@ sub create_query {
         my $q = $self->cache_method->($self, $source);
         
         # Create query
-        $query = DBIx::Custom::Query->new($q) if $q;
+        if ($q) {
+            $query = DBIx::Custom::Query->new($q);
+            $query->filters($self->filters);
+        }
     }
     
     unless ($query) {
@@ -205,7 +200,8 @@ sub create_query {
         # Cache query
         $self->cache_method->($self, $source,
                              {sql     => $query->sql, 
-                              columns => $query->columns})
+                              columns => $query->columns,
+                              tables  => $query->tables})
           if $cache;
     }
     
@@ -217,6 +213,9 @@ sub create_query {
     # Set statement handle
     $query->sth($sth);
     
+    # Set filters
+    $query->filters($self->filters);
+    
     return $query;
 }
 
@@ -304,11 +303,13 @@ sub execute{
     $query = $self->create_query($query)
       unless ref $query;
     
-    # Auto filter
+    # Applied filter
     my $filter = {};
-    my $tables = $args{table} || [];
-    $tables = [$tables]
-      unless ref $tables eq 'ARRAY';
+    my $tables = $query->tables;
+    my $arg_tables = $args{table} || [];
+    $arg_tables = [$arg_tables]
+      unless ref $arg_tables eq 'ARRAY';
+    push @$tables, @$arg_tables;
     foreach my $table (@$tables) {
         $filter = {
             %$filter,
@@ -344,13 +345,18 @@ sub execute{
     # Return resultset if select statement is executed
     if ($sth->{NUM_OF_FIELDS}) {
         
-        # Auto in filter
-        my $in_filter = {};
+        # Result in and end filter
+        my $in_filter  = {};
+        my $end_filter = {};
         foreach my $table (@$tables) {
             $in_filter = {
                 %$in_filter,
                 %{$self->{filter}{in}{$table} || {}}
-            }
+            };
+            $end_filter = {
+                %$end_filter,
+                %{$self->{filter}{end}{$table} || {}}
+            };
         }
         
         # Result
@@ -359,7 +365,8 @@ sub execute{
             filters        => $self->filters,
             filter_check   => $self->filter_check,
             default_filter => $self->{default_in_filter},
-            filter         => $in_filter || {}
+            filter         => $in_filter || {},
+            end_filter     => $end_filter || {}
         );
 
         return $result;
@@ -480,7 +487,7 @@ sub register_filter {
 sub register_tag { shift->query_builder->register_tag(@_) }
 
 our %VALID_SELECT_ARGS
-  = map { $_ => 1 } qw/table column where append relation filter query/;
+  = map { $_ => 1 } qw/table column where append relation filter query selection/;
 
 sub select {
     my ($self, %args) = @_;
@@ -496,33 +503,41 @@ sub select {
     my $tables = ref $table eq 'ARRAY' ? $table
                : defined $table ? [$table]
                : [];
-    croak qq{"table" option must be specified} unless @$tables;
-    my $columns  = $args{column} || [];
-    my $where    = $args{where} || {};
-    my $relation = $args{relation};
-    my $append   = $args{append};
-    my $filter   = $args{filter};
+    my $columns   = $args{column} || [];
+    my $selection = $args{selection} || '';
+    my $where     = $args{where} || {};
+    my $relation  = $args{relation};
+    my $append    = $args{append};
+    my $filter    = $args{filter};
     
     # SQL stack
     my @sql;
     
     push @sql, 'select';
     
-    # Column clause
-    if (@$columns) {
-        foreach my $column (@$columns) {
-            push @sql, ($column, ',');
+    if ($selection) {
+        croak qq{Can't contain "where" clause in selection}
+          if $selection =~ /\swhere\s/;
+        push @sql, $selection;
+    }
+    else {
+        # Column clause
+        if (@$columns) {
+            foreach my $column (@$columns) {
+                push @sql, ($column, ',');
+            }
+            pop @sql if $sql[-1] eq ',';
+        }
+        else { push @sql, '*' }
+        
+        # Table
+        croak qq{"table" option must be specified} unless @$tables;
+        push @sql, 'from';
+        foreach my $table (@$tables) {
+            push @sql, ($table, ',');
         }
         pop @sql if $sql[-1] eq ',';
     }
-    else { push @sql, '*' }
-    
-    # Table
-    push @sql, 'from';
-    foreach my $table (@$tables) {
-        push @sql, ($table, ',');
-    }
-    pop @sql if $sql[-1] eq ',';
     
     # Where
     my $w;
@@ -1027,8 +1042,8 @@ and implements the following new ones.
 
     $dbi->apply_filter(
         $table,
-        $column1 => {in => $infilter1, out => $outfilter1}
-        $column2 => {in => $infilter2, out => $outfilter2}
+        $column1 => {in => $infilter1, out => $outfilter1, end => $endfilter1}
+        $column2 => {in => $infilter2, out => $outfilter2, end =. $endfilter2}
         ...,
     );
 
@@ -1044,7 +1059,13 @@ arguments.
          param => {key1 => 1, key2 => 2},
          table => ['table1']
     );
-    
+
+You can use three name as column name.
+
+    1. column        : author
+    2. table.column  : book.author
+    3. table__column : book__author
+
 =head2 C<connect>
 
     my $dbi = DBIx::Custom->connect(data_source => "dbi:mysql:database=dbname",
@@ -1230,7 +1251,8 @@ This is same as L<DBI>'s C<rollback>.
                               append   => $append,
                               relation => \%relation,
                               filter   => \%filter,
-                              query    => 1);
+                              query    => 1,
+                              selection => $selection);
 
 Execute select statement.
 C<select> method have C<table>, C<column>, C<where>, C<append>,
@@ -1241,6 +1263,10 @@ C<append> is a string added at the end of the SQL statement.
 C<filter> is filters when parameter binding is executed.
 C<query> is if you don't execute sql and get L<DBIx::Custom::Query> object as return value.
 default to 0. This is experimental.
+C<selection> is string of column name and tables. This is experimental
+
+    selection => 'name, location.name as location_name ' .
+                 'from company inner join location'
 
 First element is a string. it contains tags,
 such as "{= title} or {like author}".
@@ -1309,6 +1335,14 @@ Method to set and get caches.
 
 The following tags is available.
 
+=head2 C<(experimental) table>
+
+Table tag
+
+    {table TABLE}    ->    TABLE
+
+This is used to teach what is applied table to C<execute()>.
+
 =head2 C<?>
 
 Placeholder tag.
